`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineers: Mircea Dabacan
//				  Mihai Corovei
//				  Josh Sackos
//
// 
// Create Date:    16:47:33 06/06/2012 
// Module Name:    adjustBcd - Behavioral 
// Project Name:   Hb5RefProj
// Target Devices: Nexys3
// Tool versions:  ISE 14.1
// Description: Up/Down BCD counter with predivider and with no rollover.
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module adjustBcd(
    ck,
    bitCountUp,
    bitCountDown,
    vecBcdOut
    );

	 // ===========================================================================
	 // 										Port Declarations
	 // ===========================================================================
    input ck;
    input bitCountUp;
    input bitCountDown;
    output [7:0] vecBcdOut;
	 
	 // ===========================================================================
	 // 									  Wires and Registers
	 // ===========================================================================	 
	 wire [7:0] vecBcdOut;

	 parameter intPreDiv = 24'd10000000;					// predivision factor
												// with intPreDiv = 10000000:
												// frequency(ckInt) = 100MHz/10000000 = 10Hz

	 reg [23:0] cntPreDiv = 24'h000000;		// counter for predivider
	 reg ckInt = 1'b0;
	 wire ckIntBuf;								// Internal clock
	 reg [7:0] cntUpDown = 8'h00;				// counter for PWM
	 
	 
	 // ===========================================================================
	 // 										Implementation
	 // ===========================================================================

	 // BUFG: Global Clock Buffer
    //       Spartan-6
    // Xilinx HDL Language Template, version 13.4

    BUFG BUFG_inst (
       .O(ckIntBuf), // 1-bit output: Clock buffer output
       .I(ckInt)  // 1-bit input: Clock buffer input
    );

    // End of BUFG_inst instantiation


	 // preDiv
 	 always @(posedge ck)		// clock predivision
	 begin
		 if(cntPreDiv == intPreDiv - 1'b1)
			begin
				cntPreDiv <= 1'b0;
				ckInt <= 1'b1;
			end
		 else
			begin
				cntPreDiv <= cntPreDiv + 1'b1;
				ckInt <= 1'b0;
			end
	 end


	 // UpDownCount
	 always @(posedge ckIntBuf)  // UpDown counter
	 begin
		  if(cntUpDown != 8'h99 && bitCountUp == 1'b1)  // count up
			  begin
				 if(cntUpDown[3:0] == 4'h9)
					 begin
						cntUpDown[3:0] <= 4'h0;
						cntUpDown[7:4] <= cntUpDown[7:4] + 4'h1;
					 end
				 else
					 begin
						cntUpDown[3:0] <= cntUpDown[3:0] + 4'h1;
					 end
			  end
		  else if(cntUpDown != 8'h00 && bitCountDown == 1'b1)  // count down
			  begin
				 if(cntUpDown[3:0] == 4'h0)
					begin
						cntUpDown[3:0] <= 4'h9;
						cntUpDown[7:4] <= cntUpDown[7:4] - 4'h1;
					end
				 else
					begin
						cntUpDown[3:0] <= cntUpDown[3:0] - 4'h1;
					end
			  end
			else
				begin
						//  No count
				end
	 end

	 assign vecBcdOut = cntUpDown;


endmodule
